#ifndef _CEGOAVLINDEXENTRY_H_INCLUDED_
#define _CEGOAVLINDEXENTRY_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoAVLIndexEntry.h
// ----------------
// AVL Index entry class definition
//     
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2013 Bjoern Lemke
//
// INTERFACE MODULE
//
// Class: CegoAVLIndexEntry
// 
// Description: For any kind of index object, IndexEntry objects are used as row elements
//              to refer to further entries ( parent, left, right )
//              Cego index object provide full balancing on node level, so the index entry provides
//              the required information for the balancing algorithm 
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

// cego includes
#include "CegoDataPointer.h"

class CegoAVLIndexEntry {

 public:
    
    CegoAVLIndexEntry();
    CegoAVLIndexEntry(int tid);
    ~CegoAVLIndexEntry();
    
    void initEntry(const CegoDataPointer& dp, char* p = 0, int len = 0);

    void setPtr(void* p, int len);
    void* getPtr();
    int getLen() const;
    
    void setParent(const CegoDataPointer& dp);
    CegoDataPointer getParent() const;

    void setLeftBranch(const CegoDataPointer& dp);
    CegoDataPointer getLeftBranch() const;

    void setRightBranch(const CegoDataPointer& dp);
    CegoDataPointer getRightBranch() const;

    void setHeight(char h);
    char getHeight() const;

    void setData(const CegoDataPointer& dp);
    CegoDataPointer getData() const;

    void setTID(int tid);
    int getTID() const;
        
    char* getIdxPtr();
    int getIdxLen();

    CegoAVLIndexEntry& operator = (const CegoAVLIndexEntry& ie);
    
 private:
    
    void* _pI;
    int _len;
    bool _isLocal;
    int _tid;
    char* _idxDataPtr;
};

#endif
