#ifndef _CEGOADMINTHREADPOOL_H_INCLUDED_
#define _CEGOADMINTHREADPOOL_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoAdminThreadPool.h  
// ---------------------
// Cego threadpool class definition
//                                                        
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2013 Bjoern Lemke
//
// INTERFACE MODULE
//
// Class: CegoAdminThreadPool
// 
// Description:
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

#include <lfcbase/Chain.h>
#include <lfcbase/ListT.h>
#include <lfcbase/Thread.h>
#include <lfcbase/NetHandler.h>

#include "CegoDatabaseManager.h"
#include "CegoDbThreadPool.h"
#include "CegoLogThreadPool.h"

class CegoAdminThread;

class CegoAdminThreadPool : public Thread {
    
 public:

    enum ThreadState { READY, BUSY, CONNECTED };
  
    CegoAdminThreadPool();
    CegoAdminThreadPool(int poolLimit, CegoDatabaseManager *pDBMng, CegoDbThreadPool *pDbPool, CegoLogThreadPool *pLogPool);
    ~CegoAdminThreadPool();

    int getPoolLimit() const;
    void* job(void* arg);

    void getThreadInfo(long i, long& numRequest, long& threadLoad, ThreadState& state, Chain& action);
    void setState(long i, ThreadState state);
    void setTid(long i, long tid);
    void incNumRequest(long i);

    void addThreadIdle(long i, long usec);

    void dispatch();

    NetHandler* nextRequest();

    bool isTerminated();

    void getLockStat(Chain& lockName, long& lockCount, long &numRdLock, long &numWrLock, long &sumRdDelay, long &sumWrDelay);

 private:

    void lockQueue();
    void unlockQueue();
	
    long* _threadId;
    long* _numRequest;
    ThreadState* _threadState;

    long* _threadIdle[THRMNG_NUMLOADSAMPLE];
    long* _threadLoad;
    int _samplePos;

    CegoAdminThread** _threadList;
    

    int _poolLimit;
    Chain _adminHostName;
    int _adminPortNo;

    bool _terminated;
    bool _joined;

    ListT<NetHandler*> _requestQueue;

    CegoDatabaseManager* _pDBMng;

    unsigned long _modId;
};

#endif














