#ifndef _CEGOBTREEMANAGER_H_INCLUDED_
#define _CEGOBTREEMANAGER_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoBTreeManager.h
// ------------------
// Cego BTree+ manager class definition
//                                                         
// Design and Implementation by Bjoern Lemke               
//     
// (C)opyright 2000-2013 Bjoern Lemke
//
// INTERFACE MODULE
//
// Class: CegoBTreeManager
// 
// Description: This class implements the btree algorithm to be used as an alternative index object.
//
// Status: 2.16
//
///////////////////////////////////////////////////////////////////////////////

// base includes
#include <lfcbase/Chain.h>
#include <lfcbase/ListT.h>

// cego includes
#include "CegoDataType.h"
#include "CegoField.h"
#include "CegoDataPointer.h"
#include "CegoSystemObject.h"
#include "CegoObjectManager.h"
#include "CegoBTreeNode.h"
#include "CegoBTreeValue.h"

class CegoBTreeManager {

public:
    
  CegoBTreeManager(CegoObjectManager* pObjMng, CegoBTreeObject *pBTO);
    ~CegoBTreeManager();

    void insertBTree(const CegoDataPointer& dp,
		     const CegoBTreeValue &iv,
		     int tid);

    void deleteBTree(const CegoBTreeValue &iv,
		     const CegoDataPointer& ddp);

    void insertBTree(const CegoDataPointer& sysEntry,
		      const CegoDataPointer& dp, 
		      const CegoBTreeValue &iv,
		      int tid);

    void deleteBTree(const CegoDataPointer& sysEntry, 
		     const CegoBTreeValue &iv,
		     const CegoDataPointer& dp);

    void fixRoot();
    void unfixRoot();


    void dumpBTree();
    void dumpNode(int level, int fileId, int pageId);

    

    int getNumPages();
    int freeBTree();

    
    char checkIndex(int tabSetId, const Chain& indexName, CegoObject::ObjectType idxType);

private:

    int countNodePages(int fileId, int pageId, int& firstLeafFileId, int& firstLeafPageId, bool& isFirst);
    int countLeafPages(int firstLeafFileId, int firstLeafPageId);
    
    int freeNodePages(int fileId, int pageId, int& firstLeafFileId, int& firstLeafPageId, bool& isFirst);
    int freeLeafPages(int firstLeafFileId, int firstLeafPageId);


    CegoBTreeObject* _pBTO;
    CegoObjectManager* _pObjMng;
    CegoDatabaseManager* _pDBMng;
    
    CegoObject::ObjectType _btreeType;
    ListT<CegoField> _btreeSchema;
    Chain _btreeName;
    int _keyLen;
    int _tabSetId;

    CegoBufferPage _rootBP;
    bool _rootFixed;

    unsigned long _modId;
};

#endif




