#ifndef _CEGOBUFFERPOOL_H_INCLUDED_
#define _CEGOBUFFERPOOL_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoBufferPool.h
// ----------------
// Cego BufferPool class definition
//                                                         
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2013 Bjoern Lemke
//
// INTERFACE MODULE
//
// Class: CegoBufferPool
// 
// Description: Cego buffer pool
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

#include <lfcbase/Chain.h>
#include <lfcbase/NanoTimer.h>

#include "CegoXMLSpace.h"
#include "CegoLogManager.h"
#include "CegoFileHandler.h"
#include "CegoBufferPage.h"
#include "CegoBufferPoolEntry.h"
#include "CegoLockHandler.h"

class CegoBufferPool : public CegoLogManager {
    
public:
    
    enum FixMode { SYNC, NOSYNC, PERSISTENT };
    
    CegoBufferPool(const Chain& xmlDef, const Chain& logFile, const Chain& progName);
    ~CegoBufferPool();
    
    void initPool(long numPages);
    void removePool();
    
    void attachPool();
    void detachPool();
    
    void bufferFix(CegoBufferPage &bp, int tabSetId, int fileId, int pageId, FixMode m, CegoLockHandler *pLockHandle, int numTry = 0);
    void emptyFix(CegoBufferPage &bp, int tabSetId, FixMode m, CegoFileHandler::FileType ft, CegoLockHandler *pLockHandle, bool doAppend = false);

    void bufferUnfix(CegoBufferPage &bp, bool isDirty, CegoLockHandler *pLockHandle);
    void bufferRelease(CegoBufferPage &bp, CegoLockHandler *pLockHandle);

    int writeCheckPoint(int tabSetId, bool switchLog, const Chain& escCmd, int timeout, CegoLockHandler *pLockHandle);
    void writeAndRemoveTabSet(int tabSetId, CegoLockHandler *pLockHandle);

    int uptime() const;
    void poolInfo(int& pageSize,
		  long& numTotal,
		  long& numUsed,
		  long& numFree,
		  long& numDirty,
		  long& numFixes,
		  long& numPersistent,
		  long& numNoSync,
		  long& numDiskRead,
		  long& numDiskWrite,
		  double& hitRate,
		  long& avgReadDelay,
		  long& avgWriteDelay,
		  long& curFixCount,
		  long& maxFixCount,
		  int& statStart,
	          int& uptime) const;

    void getPoolEntryList(ListT<CegoBufferPoolEntry>& entryList);

    void resetStats();

    void printPool();

    Chain getDBName();

private:

    int calcHash(int fileId, int pageId);

    typedef struct BufferPoolHead {
	int numPages;
    } BufferPoolHead;
    
    enum OccState { NOT_OCCUPIED, WRITE_ON_SYNC, WRITE_ON_DIRTY, PERSISTENT_OCCUPIED };
    
    typedef struct BufferHead {
	OccState isOccupied;
	char isDirty;
	int numFixes;
	int tabSetId;
	int fileId;
	int pageId;
	int fixStat;
    } BufferHead;

    void logBM(int tabSetId, int fileId, unsigned* fbm, int fbmSize);
    
    Chain _dbName;
    long _numPages;
    int _key;
    
    void *_pBufPool;

    long _fixCount;
    long _numDiskRead;    
    long _numDiskWrite;
    long _avgReadDelay;
    long _avgWriteDelay;

    NanoTimer _diskReadTimer;
    NanoTimer _diskWriteTimer;

    int _poolStart;
    int _statStart;
    int _maxFixTries;

    unsigned long _modId;
};



#endif




