#ifndef _CEGOFUNCTION_H_INCLUDED_
#define _CEGOFUNCTION_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoFunction.h
// --------------
// Cego internal sql function class definition
//     
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2013 Bjoern Lemke
//
// INTERFACE MODULE
//
// Class: CegoFunction
// 
// Description: 
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

#include <lfcbase/Chain.h>
#include <lfcbase/ListT.h>

#include "CegoAttrDesc.h"
#include "CegoField.h"
#include "CegoFieldValue.h"
#include "CegoProcBlock.h"


class CegoDistManager;
class CegoTableManager;
class CegoAggregation;
class CegoExpr;
class CegoDatabaseFormater;

class CegoFunction {
    
 public:
    
    enum FunctionType {  TRIM, 
			 RTRIM,
			 LTRIM,
			 ROUND,
			 DATE2STR, 
			 DATE2INT, 
			 INT2DATE, 
			 LEFT,
			 RIGHT,
			 GETPOS,
			 SUBSTR,
			 REPLACE,
			 LENGTH,
			 LOWER,
			 UPPER,
			 TRUNC,
			 STR2INT,
			 STR2LONG,
			 STR2DATE,
			 RANDSTR,
			 RANDINT,
			 MOD,
			 DIV,
			 POWER,
			 BITAND,
			 BITOR,
			 BITXOR,
			 BLOBSIZE,
			 NEXTCOUNT,
			 SETCOUNT,
			 USERDEFINED }; 



    CegoFunction(FunctionType funcType);
    CegoFunction(FunctionType funcType, ListT<CegoExpr*>& exprList);

    CegoFunction(char* buf, CegoDistManager *pTabMng, int tabSetId);
    CegoFunction(CegoDistManager* pTabMng, int tabSetId, CegoFunction::FunctionType type);
    CegoFunction(CegoDistManager* pTabMng, int tabSetId, const Chain& funcName, ListT<CegoExpr*>& exprList);
    
    CegoFunction(Element* pFunctionElement, CegoDistManager *pGTM);

    ~CegoFunction();

    CegoDataType getReturnType() const;
    int getReturnTypeLen() const;

    ListT<CegoExpr*>& getExprList();
    ListT<CegoField> getFieldList() const;
    void setFieldListArray(ListT<CegoField> *fla, int size);
    void setBlock(CegoProcBlock *pBlock);

    const FunctionType getType() const;

    void setExprList(ListT<CegoExpr*>& exprList);

    void clearAttrCache();

    ListT<CegoAttrDesc*> getAttrRefList() const;
    int evalReferences(CegoContentObject *pCO, const ListT<CegoField>& fl);

    void setCounterId(const Chain& counterId);
    void setCounterExpr(CegoExpr* pCounterExpr);

    ListT<CegoAggregation*> getAggregationList();
    
    CegoFieldValue evalFieldValue() const;
    Chain getId() const;
    Chain toChain(const Chain& indent = Chain("")) const;
    Chain dbFormat(CegoDatabaseFormater *pForm) const;
    
    Element* toElement() const;
    void fromElement(Element *pFunctionElement, CegoDistManager *pGTM);

    void encode(char *buf);
    void decode(char *buf, CegoDistManager* pGTM, int tabSetId);
    int getEncodingLength() const;

    CegoFunction* clone(bool isAttrRef = false);

    CegoFunction& operator = ( const CegoFunction& f);

    friend ostream& operator << (ostream& s, const CegoFunction& f);
      
 private:

    Chain getFuncString() const;
    
    CegoDistManager* _pTabMng;
    CegoProcBlock* _pBlock;
    Chain _funcName;

    Chain _counterId;
    
    FunctionType _type;
    ListT<CegoExpr*> _exprList;
    int _tabSetId;
};

#endif




