#ifndef _CEGOGROUPSPACE_H_INCLUDED_
#define _CEGOGROUPSPACE_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoGroupSpace.h
// -----------------
// Cego group space class definition
//     
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2013 Bjoern Lemke
//
// INTERFACE MODULE
//
// Class: CegoGroupSpace
// 
// Description: Utility class for grouping tuples
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

#include <lfcbase/Chain.h>
#include <lfcbase/ListT.h>

#include "CegoDefs.h"
#include "CegoAttrDesc.h"
#include "CegoTableManager.h"
#include "CegoGroupCursor.h"
#include "CegoGroupNode.h"

class CegoGroupSpace {
    
 public:

    CegoGroupSpace();
    ~CegoGroupSpace();
       
    void initGroupSpace(const ListT<CegoField>& schema, ListT<CegoAggregation*>& aggList, long maxGroupSize);

    void insertTuple(const ListT<CegoField>& groupKey, ListT<CegoField>& groupTuple);

    void resetGroupSpace();

    ListT<CegoField> getSchema();

    CegoGroupCursor* getCursor();

    long numAllocated() const;

 private:

    ListT<CegoField> initGrouping(ListT<CegoField>& dataTuple);
    ListT<CegoField> aggGrouping(ListT<CegoField>& aggValues, ListT<CegoField>& dataTuple);

    CegoFieldValue getValueForAgg(CegoAggregation* pAgg, ListT<CegoField>& dataTuple);

    int _groupingOffset;

    ListT<CegoField> _groupSchema;

    CegoAggregation* _pCountAgg;

    ListT<CegoAggregation*> _aggList;
    ListT<CegoField> _aggSchema;

    AVLTreeT<CegoGroupNode>* _pAVL;

    long _maxOrderSize;
    long _orderSize;

};

#endif




