#ifndef _CEGOORDERSPACE_H_INCLUDED_
#define _CEGOORDERSPACE_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoOrderSpace.h
// -----------------
// Cego order space class definition
//     
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2013 Bjoern Lemke
//
// INTERFACE MODULE
//
// Class: CegoOrderSpace
// 
// Description: Cego order space
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

// base includes
#include <lfcbase/Chain.h>

// cego includes
#include "CegoDefs.h"
#include "CegoAttrDesc.h"
#include "CegoOrderNode.h"
#include "CegoOrderCursor.h"
#include "CegoQueryHelper.h"

// forward declaration
class CegoTableManager;

class CegoOrderSpace {
    
 public:

    CegoOrderSpace();
    ~CegoOrderSpace();
       
    void initOrderSpace(ListT<CegoExpr*> *pOrderList, ListT<CegoOrderNode::Ordering> *pOrderOptList, long maxOrderSize);
    void insertTuple(ListT<CegoField>& orderKey, ListT<CegoField>& orderTuple);

    void resetOrderSpace();

    CegoOrderCursor* getCursor();

    long numAllocated() const;
 
 private:

    AVLTreeT<CegoOrderNode>* _pAVL;

    void setAggregationValue(CegoExpr *pExpr, ListT<CegoField>& fl);

    ListT<CegoExpr*>* _pOrderList;
    ListT<CegoOrderNode::Ordering>* _pOrderOptList;
    ListT<CegoField> _orderSchema;

    long _maxOrderSize;
    long _orderSize;

};

#endif




